﻿/*  XNA Line Drawer Class
 *  
 *  Author: Gregory Walek
 * 
 *  Based on the Code Located at 
 *  http://www.xnawiki.com/index.php?title=Drawing_2D_lines_without_using_primitives
 */
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace LineDraw
{
    public static class LineDrawer
    {
        static Texture2D blank;

        private static void InitLineDrawer(GraphicsDevice GraphicsDevice)
        {
            blank = new Texture2D(GraphicsDevice, 1, 1, false, SurfaceFormat.Color);
            blank.SetData(new[] { Color.White });
        }

        public static void DrawLine(SpriteBatch spriteBatch, float width, Color color, Vector2 point1, Vector2 point2)
        {
            if (blank == null)
            {
                InitLineDrawer(spriteBatch.GraphicsDevice);
            }

            float angle = (float)Math.Atan2(point2.Y - point1.Y, point2.X - point1.X);
            float length = Vector2.Distance(point1, point2);

            Vector2 offset = new Vector2(0.0f, 0.5f);

            spriteBatch.Draw(blank, point1, null, color,
                       angle, offset, new Vector2(length, width),
                       SpriteEffects.None, 0);
        }

        public static void DrawLine(SpriteBatch spriteBatch, float width, Color color, Line line)
        {
            DrawLine(spriteBatch, width, color, line.startPoint, line.endPoint); 
        }

        public static void DrawLine(SpriteBatch spriteBatch, Line line)
        {
            line.Draw(spriteBatch); 
        }

        public static void DrawLine2(SpriteBatch spriteBatch, float width, Color color, Vector2 point1, Vector2 point2)
        {
            if (blank == null)
            {
                InitLineDrawer(spriteBatch.GraphicsDevice);
            }

            float angle = (float)Math.Atan2(point2.Y - point1.Y, point2.X - point1.X);
            float length = Vector2.Distance(point1, point2);

            spriteBatch.Draw(blank, point1, null, color,
                       angle, Vector2.Zero, new Vector2(length, width),
                       SpriteEffects.None, 0);
        }

        public static void DrawLine2(SpriteBatch spriteBatch, float width, Color color, Line line)
        {
            DrawLine2(spriteBatch, width, color, line.startPoint, line.endPoint);
        }

        public static void DrawRectangle(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec)
        {
            DrawLine(spriteBatch, width, color,     new Vector2( (Rec.X), (Rec.Y) ) ,                               new Vector2( (Rec.X + Rec.Width), (Rec.Y) ) );
            DrawLine(spriteBatch, width, color,     new Vector2( (Rec.X + Rec.Width), Rec.Y ),                      new Vector2( (Rec.X + Rec.Width), (Rec.Y + Rec.Height) ) );
            DrawLine(spriteBatch, width, color,     new Vector2( (Rec.X + Rec.Width), (Rec.Y + Rec.Height) ) ,      new Vector2( (Rec.X ), (Rec.Y + Rec.Height) ) );
            DrawLine(spriteBatch, width, color,     new Vector2( (Rec.X ), (Rec.Y + Rec.Height) ) ,                 new Vector2( (Rec.X ), (Rec.Y) ) );
        }

        public static void DrawRectangle(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec, float roation)
        {
            Vector2 Center = new Vector2(Rec.Center.X, Rec.Center.Y);

            DrawRectangle(spriteBatch, width, color, Rec, roation, Center); 
            
        }

        public static void DrawRectangle(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec, float Roation, Vector2 Center)
        {
            DrawLine(spriteBatch, width, color, RotatePoint(Center, Roation, new Vector2((Rec.X), (Rec.Y))), LineDrawer.RotatePoint(Center, Roation, new Vector2((Rec.X + Rec.Width), (Rec.Y))));
            DrawLine(spriteBatch, width, color, RotatePoint(Center, Roation, new Vector2((Rec.X + Rec.Width), Rec.Y)), LineDrawer.RotatePoint(Center, Roation, new Vector2((Rec.X + Rec.Width), (Rec.Y + Rec.Height))));
            DrawLine(spriteBatch, width, color, RotatePoint(Center, Roation, new Vector2((Rec.X + Rec.Width), (Rec.Y + Rec.Height))), LineDrawer.RotatePoint(Center, Roation, new Vector2((Rec.X), (Rec.Y + Rec.Height))));
            DrawLine(spriteBatch, width, color, RotatePoint(Center, Roation, new Vector2((Rec.X), (Rec.Y + Rec.Height))), LineDrawer.RotatePoint(Center, Roation, new Vector2((Rec.X), (Rec.Y))));
        }

        public static void DrawSolidRectangle(SpriteBatch spriteBatch, Color color, Rectangle Rec)
        {
            float width = Rec.Width; 
            DrawLine(spriteBatch, width, color, new Vector2(Rec.Center.X, Rec.Bottom), new Vector2(Rec.Center.X, Rec.Top));
           
        }

        public static void DrawSolidRectangle(SpriteBatch spriteBatch, Color color, Rectangle Rec, float Roation, Vector2 Center)
        {
            float width = Rec.Width;
            DrawLine(spriteBatch, width, color, RotatePoint(Center, Roation, new Vector2(Rec.Center.X, Rec.Bottom)), RotatePoint(Center, Roation, new Vector2(Rec.Center.X, Rec.Top)));

        }

        public static void DrawSolidRectangle(SpriteBatch spriteBatch, Color color, Rectangle Rec, float Roation)
        {
            DrawSolidRectangle(spriteBatch, color, Rec, Roation, new Vector2(Rec.Center.X, Rec.Center.Y));
        }

        public static void DrawCircle(SpriteBatch spriteBatch, float width, Color color, Vector2 Position, float Radius, int Sides)
        {
            Vector2 Starting = new Vector2(Radius, 0);
            Vector2 Next;
            Vector2 Previous = Starting;
            float degrees = 360 / Sides;

            //Console.WriteLine(degrees); 

            if (Sides < 3)
            {
                Sides = 36;
            }

            for (int i = 1; i < Sides;  i++)
            {
                /// This is drawing the circle Counter Clock Wise) 

                Next = AngleToV2((i * degrees), Radius);
                LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Next);

                // Debug Line
                //Console.WriteLine(i+"*" + (i * degrees) + "::" + Previous + "::" + Next); 

                Previous = Next;

            }

            LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Starting);

        }

        public static void DrawArc(SpriteBatch spriteBatch, float width, Color color, Vector2 Position, float OuterRadius, float InnerRadius, float Arc, float Rotation, int Sides)
        {
            float degrees = Arc / Sides;
          
            Vector2 OuterStart  = AngleToV2(Rotation - Arc / 2, OuterRadius);
            Vector2 OuterEnd    = AngleToV2(Rotation + Arc / 2, OuterRadius);
            Vector2 InnerStart  = AngleToV2(Rotation - Arc / 2, InnerRadius);
            Vector2 InnerEnd    = AngleToV2(Rotation + Arc / 2, InnerRadius);

            if (Sides < 3)
            {
                Sides = 12;
            }

            // Draw the Side of the Arc
            LineDrawer.DrawLine(spriteBatch, width, color, Position + OuterStart, Position + InnerStart);
            LineDrawer.DrawLine(spriteBatch, width, color, Position + OuterEnd, Position + InnerEnd);

            if (InnerRadius > 0)
            {
                LineDrawer.DrawLine(spriteBatch, width, color, Position + InnerStart, Position + InnerEnd);
            }

            Vector2 Next;
            Vector2 Previous = OuterStart; 

            for (int i = 1; i < Sides; i++)
            {
                /// This is drawing the circle Counter Clock Wise) 

                Next = AngleToV2((i * degrees) + (Rotation - Arc / 2), OuterRadius);
                LineDrawer.DrawLine(spriteBatch, width, color, Position + Previous, Position + Next);

                // Debug Line
                //Console.WriteLine(i+"*" + (i * degrees) + "::" + Previous + "::" + Next); 

                Previous = Next;

            }
            LineDrawer.DrawLine(spriteBatch, width, color, Position + Previous, Position + OuterEnd);
      
            
        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Vector2 Position, Vector2 Axis, int Sides)
        {
            Vector2 Starting = new Vector2(Axis.X, 0);
            Vector2 Next;
            Vector2 Previous = Starting;
            float degrees = 360 / Sides;

            //Console.WriteLine(degrees); 

            if (Sides < 3)
            {
                Sides = 36;
            }

            for (int i = 1; i < Sides; i++)
            {
                /// This is drawing the circle Counter Clock Wise) 

                Next = AngleToEllipseV2((i * degrees), 1.0f, Axis);
                LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Next);

                // Debug Line
                //Console.WriteLine(i+"*" + (i * degrees) + "::" + Previous + "::" + Next); 

                Previous = Next;

            }

            LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Starting);

        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Vector2 Position, Vector2 Axis, float Rotation, int Sides)
        {
            DrawEllipse(spriteBatch, width, color, Position, Axis, Rotation, Vector2.Zero, Sides);
        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Vector2 Position, Vector2 Axis, float Rotation, Vector2 Center, int Sides)
        {
            Vector2 RoationCenter = Vector2.Zero;
            if (Center != Vector2.Zero)
                RoationCenter = Center - Position;

            Vector2 Starting = RotatePoint(RoationCenter, Rotation, new Vector2(Axis.X, 0));
            Vector2 Next;
            Vector2 Previous = Starting;
            float degrees = 360 / Sides;

            //Console.WriteLine(degrees); 

            if (Sides < 3)
            {
                Sides = 36;
            }

            for (int i = 1; i < Sides; i++)
            {
                /// This is drawing the circle Counter Clock Wise) 

                Next = RotatePoint(RoationCenter, Rotation, AngleToEllipseV2((i * degrees), 1.0f, Axis));
                LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Next);

                // Debug Line
                //Console.WriteLine(i+"*" + (i * degrees) + "::" + Previous + "::" + Next); 
 
                Previous = Next;

            }

            LineDrawer.DrawLine2(spriteBatch, width, color, Position + Previous, Position + Starting);

        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec, int sides)
        {
            Vector2 Center = Vector2.Zero;
            Vector2 Axis = Vector2.Zero;

            Center.X = Rec.Center.X;
            Center.Y = Rec.Center.Y;
            Axis.X = Rec.Width / 2.0f;
            Axis.Y = Rec.Height / 2.0f;

            LineDrawer.DrawEllipse(spriteBatch, width, color,Center, Axis, sides); 
        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec, float roation, int sides)
        {
            DrawEllipse(spriteBatch, width, color, Rec, roation, Vector2.Zero, sides);
        }

        public static void DrawEllipse(SpriteBatch spriteBatch, float width, Color color, Rectangle Rec, float roation, Vector2 RotCenter, int Sides)
        {
            Vector2 DrawCenter = Vector2.Zero;
            Vector2 Axis = Vector2.Zero;
            
            DrawCenter.X = Rec.Center.X;
            DrawCenter.Y = Rec.Center.Y;
            Axis.X = Rec.Width / 2.0f;
            Axis.Y = Rec.Height / 2.0f;
           
            DrawEllipse(spriteBatch, width, color, DrawCenter, Axis, roation, RotCenter, Sides);
        }

        public static Vector2 AngleToV2(float angle, float length)
        {
            Vector2 direction = Vector2.Zero;
            direction.X = (float)Math.Cos(MathHelper.ToRadians(angle)) * length;
            direction.Y = (float)Math.Sin(MathHelper.ToRadians(angle)) * length;
            return direction;
        }

        public static Vector2 AngleToEllipseV2(float angle, float length, Vector2 Axis)
        {
            Vector2 direction = Vector2.Zero;
            direction.X = (float)Math.Cos(MathHelper.ToRadians(angle)) * Axis.X  * length;
            direction.Y = (float)Math.Sin(MathHelper.ToRadians(angle)) * Axis.Y * length;
            return direction;
        }

        public static Vector2 AngleToEllipseV2(float angle, Vector2 Axis)
        {
            return AngleToEllipseV2(angle, 1f, Axis); 
        }

        public static float V2ToAngle(Vector2 startPoint, Vector2 endPoint)
        {
            float xDiff = startPoint.X - endPoint.X;
            float yDiff = startPoint.Y - endPoint.Y;
            return (float)(Math.Atan2(yDiff, xDiff) * 180.0 / Math.PI); 
        }


        public static Vector2 RotatePoint(Vector2 Center, float angle, Vector2 pointIN)
        {
            Vector2 p = new Vector2(pointIN.X, pointIN.Y);
            float s = (float)Math.Sin(MathHelper.ToRadians(angle));
            float c = (float)Math.Cos(MathHelper.ToRadians(angle));

            // translate point back to origin:
            p.X -= Center.X;
            p.Y -= Center.Y;

            // rotate point
            float xnew = p.X * c - p.Y * s;
            float ynew = p.X * s + p.Y * c;

            // translate point back:
            p.X = xnew + Center.X;
            p.Y = ynew + Center.Y;

            return p; 
        }
    }
 
}